# Bit_logger.py - 21.06.2021

from machine import Pin, Timer, lightsleep
import utime as time
import array as arr

class Bit_Logger(object):
    def __init__(self, pin, mode=(Pin.IRQ_FALLING | Pin.IRQ_RISING), wait_after_start=500):
        self.ir_pin = Pin(pin, Pin.IN, Pin.PULL_UP)
        self.wait_after_start = wait_after_start
        if mode == (Pin.IRQ_FALLING | Pin.IRQ_RISING):
            self.mode = mode
            self.ir_pin.irq(self.check, trigger=(Pin.IRQ_FALLING | Pin.IRQ_RISING))
        else:
            self.mode = (Pin.IRQ_FALLING) 
            self.ir_pin.irq(self.check, trigger=(Pin.IRQ_FALLING)) 

        self.arr_size = 200  
        self.cnt = 0
        self.last = time.ticks_us()
        # Index = Bits 0xFF und Signal-Level (0 = Low, 1 = High), abgelegt in Bit 0x100
        self.idx = arr.array("H", [0] * self.arr_size)
        # Dauer von Pulse und Pause in us
        self.dur = arr.array("H", [0] * self.arr_size)
        self.tim = Timer()

    def check(self, _=None):
        if self.cnt == 0:
            self.last = time.ticks_us()
            # nach "self.wait_after_time" die Ausgabe anzeigen
            self.tim.init(period = self.wait_after_start, mode=Timer.ONE_SHOT, callback=self.print_log)

        else:
            now = time.ticks_us()
            pulse = time.ticks_diff(now, self.last)
            
            # in self.idx[] den Index in den unteren beiden Bytes speichernm, den Level im Bit 0x10000
            self.idx[self.cnt] = self.cnt | (0 if self.ir_pin.value() else 0x1000)
            self.dur[self.cnt] = time.ticks_diff(now, self.last)
            self.last = now
        
        self.cnt += 1
        if self.cnt > self.arr_size - 3:
            self.ir_pin.irq(None)
            self.tim.deinit()
            print("Buffer full, logging stopped at Bit: {}\n".format((self.cnt-2) // 2))
            self.print_log()

    def print_log(self, _=None):
        # IRQ disable, damit während der Ausgabe keine weiteren Daten gelogged werden.
        self.ir_pin.irq(None)
        i = 1
        if self.cnt > 0:
            while(self.idx[i]):
                if self.mode == (Pin.IRQ_FALLING | Pin.IRQ_RISING): 
                    # der Index steht in den unteren beiden Bytes in self.idx[], der Bitstatus im Bit 0x100 !
                    print("Bit {:2} - {} Pulse: {:5}".format(((self.idx[i] & 0xFFF) - 2) // 2, \
                                                           (1 if self.idx[i] & 0x1000 else 0), self.dur[i]), end = "")
                    i += 1
                    print(" - {} Pause: {:5} - Sum: {:6}".format((1 if self.idx[i]& 0x1000 else 0), \
                                                            self.dur[i], self.dur[i] + self.dur[i-1]))

                else:
                    print("Bit {:2} Pulse+Pause: {:5}".format((self.idx[i] & 0xFFF) - 2, self.dur[i]))
                i += 1
            print("---\n")

                  
        # Index zurücksetzen
        for i in range(self.arr_size):
            self.idx[i] = 0
            self.dur[i] = 0
        self.cnt = 0
        
        # Interrupt wieder enablen
        self.ir_pin.irq(self.check, trigger=self.mode) 

if __name__ == "__main__":
    # Hier ist der TSOP angechlossen, der Ruhepegel ist high
    ir_pin = 17
    
    # Mode : (Pin.IRQ_FALLING | Pin.IRQ_RISING) [default] oder (Pin.IRQ_RISING) 
    irq_mode = (Pin.IRQ_FALLING | Pin.IRQ_RISING)
    #irq_mode = (Pin.IRQ_RISING)
    
    ir = Bit_Logger(pin = ir_pin, mode = irq_mode)
    print("IR-Remote-Timing-Checker")
    print("Press IR-Remote and wait ... \n")
    
    while True:
        lightsleep()
